from django.views.generic.base import TemplateView
from rest_framework import viewsets, permissions, pagination
from rest_framework.schemas import AutoSchema
from rest_framework.response import Response
from rest_framework.compat import coreapi
from django.core.exceptions import ObjectDoesNotExist
from rest_framework.authtoken.serializers import AuthTokenSerializer
from rest_framework.authtoken.models import Token
from django.db import IntegrityError
from . import serializers, models


class CustomViewSchema(AutoSchema):
    def get_manual_fields(self, path, method):
        extra_fields = []
        if path.endswith('/register/'):
            extra_fields = [
                coreapi.Field("name", required=True),
                coreapi.Field("email", required=True),
                coreapi.Field("password", required=True),
            ]

        manual_fields = super().get_manual_fields(path, method)
        return manual_fields + extra_fields

class HasPermPage(permissions.BasePermission):
    def has_permission(self, request, view):
        return request.user.has_perm_page(view.basename) if request.user.is_authenticated else False

# ===================================================
class RegisterViewSet(viewsets.ViewSet):
    schema = CustomViewSchema()
    permission_classes = (permissions.AllowAny,)

    def create(self, request):
        """ Register. """
        success = True
        error = ''
        try:
            new_user = models.User.objects.create_user(
                email = request.data.get('email'),
                password = request.data.get('password'),
            )
            new_user.name = request.data.get('name')
            try:
                # new_user.group = models.Group.objects.get(name='free')
                print(new_user)
                new_user.save()
            except ObjectDoesNotExist:
                new_user.delete()
                error = 'Please contact our support team.'

            serializer = AuthTokenSerializer(data={'username': request.data.get('email'), 'password': request.data.get('password')}, context={'request': request})
            serializer.is_valid(raise_exception=True)
            user = serializer.validated_data['user']
            token, created = Token.objects.get_or_create(user=user)
            token = token.key
            message = {
                'type': 'success',
                'message': 'Registered successfully.'
            }
        except IntegrityError:
            token = ''
            success = False
            message = {
                'type': 'danger',
                'message': 'Exist email address provided.'
            }
            error = 'Exist email address provided.'

        return Response({ 
            'success': success, 
            'token': token,
            'error': error,
            'message': message,
        })

class UserViewSet(viewsets.ModelViewSet):
    """ User. """
    queryset = models.User.objects.all().order_by('-date_joined')
    serializer_class = serializers.UserSerializer
    permission_classes = [HasPermPage]
    http_method_names = ['get', 'post', 'put', 'delete']

class GroupViewSet(viewsets.ModelViewSet):
    """ Group. """
    queryset = models.Group.objects.all()
    serializer_class = serializers.GroupSerializer
    permission_classes = [HasPermPage]
    http_method_names = ['get', 'post', 'put', 'delete']


from django.urls import path
from django.conf.urls import include
from rest_framework import routers
from rest_framework_swagger.views import get_swagger_view
from rest_framework.authtoken.views import obtain_auth_token
from . import views

router = routers.SimpleRouter()
router.register(r'user', views.UserViewSet, basename='user')
router.register(r'group', views.GroupViewSet, basename='group')
router.register(r'register', views.RegisterViewSet, basename='register')

schema_view = get_swagger_view(title='Crazy Coding API')

urlpatterns = [
    path('swagger', schema_view),
    path('', include(router.urls)),

    path('auth/', include('rest_framework.urls', namespace='rest_framework')),
    path('auth-token/', obtain_auth_token)
]


# Generated by Django 3.1.1 on 2020-09-23 07:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=254)),
                ('permission_page', models.TextField(null=True)),
            ],
            options={
                'db_table': 'group',
            },
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('name', models.CharField(max_length=254, null=True)),
                ('password', models.CharField(max_length=128)),
                ('is_admin', models.BooleanField(default=False)),
                ('is_active', models.BooleanField(default=True)),
                ('date_joined', models.DateField(auto_now=True)),
                ('group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='api_v1.group')),
            ],
            options={
                'db_table': 'user',
            },
        ),
    ]



from . import models
from rest_framework import serializers
from rest_framework.authtoken.models import Token


class CustomSerializer(serializers.ModelSerializer):
    def get_field_names(self, declared_fields, info):
        expanded_fields = super(CustomSerializer, self).get_field_names(declared_fields, info)

        if getattr(self.Meta, 'extra_fields', None):
            return expanded_fields + self.Meta.extra_fields
        else:
            return expanded_fields

# ===================================================
class GroupSerializer(CustomSerializer):
    class Meta:
        model = models.Group
        fields = '__all__'

class TokenSerializer(CustomSerializer):
    class Meta:
        model = Token
        fields = '__all__'

class UserSerializer(CustomSerializer):
    group = GroupSerializer(read_only=True)
    token = TokenSerializer(read_only=True)

    class Meta:
        model = models.User
        fields = '__all__'
        extra_fields = ['token']


from django.db import models
from django.contrib.auth.models import BaseUserManager, AbstractBaseUser
from rest_framework.authtoken.models import Token
from rest_framework import serializers


class CustomModelBase(models.base.ModelBase):
    def __new__( cls, name, bases, attrs, **kwargs ):
        if name != "CustomModel":
            class MetaB:
                db_table = "" + name.lower()

            attrs["Meta"] = MetaB

        return super().__new__( cls, name, bases, attrs, **kwargs )

class CustomModel(models.Model, metaclass=CustomModelBase):
    class Meta:
        abstract = True

# ===================================================
class UserManager(BaseUserManager):
    def create_user(self, email, password=None):
        if not email:
            raise ValueError('Users must have an email address')

        user = self.model(
            email=self.normalize_email(email)
        )

        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password=None):
        user = self.create_user(
            email,
            password=password
        )
        user.is_admin = True
        user.save(using=self._db)
        return user

class User(CustomModel, AbstractBaseUser):
    email = models.EmailField(max_length=254, unique=True)
    name = models.CharField(max_length=254, null=True)
    password = models.CharField(max_length=128)
    is_admin = models.BooleanField(default=False)
    is_active = models.BooleanField(default=True)
    date_joined = models.DateField(auto_now=True)
    group = models.ForeignKey('Group', on_delete=models.PROTECT, null=True)

    objects = UserManager()

    USERNAME_FIELD = 'email'

    def has_perm_page(self, page):
        return True if (self.group and page in self.group.permission_page) or (self) else False

    @property
    def token(self):
        return Token.objects.get(user_id=self)

class Group(CustomModel):
    name = models.CharField(max_length=254)
    permission_page = models.TextField(null=True)


from django.test import TestCase
from rest_framework.authtoken.serializers import AuthTokenSerializer
from . import models


class UserGroupTestCase(TestCase):
    def setUp(self):
        admin_group = models.Group.objects.create(name="admin", permission_page="user,group")
        client_group = models.Group.objects.create(name="client", permission_page="")

        admin_user = models.User.objects.create_user(email="admin1@test.com", password="admin")
        admin_user.name = "Administrator1"
        admin_user.group = admin_group
        admin_user.save()
        client_user = models.User.objects.create_user(email="client1@test.com", password="client")
        client_user.name = "Client1"
        client_user.group = client_group
        client_user.save()

    def test_group_created(self):
        """[Testing] Group Model Create"""
        self.assertEqual(models.Group.objects.get(name="admin").name, "admin")
        self.assertEqual(models.Group.objects.get(name="client").name, "client")

    def test_user_created(self):
        """[Testing] User Model Create"""
        self.assertEqual(models.User.objects.get(name="Administrator1").name, "Administrator1")
        self.assertEqual(models.User.objects.get(name="Client1").name, "Client1")

    def test_user_group_relations(self):
        """[Testing] Relation with User and Group models"""
        admin_group = models.Group.objects.get(name="admin")
        admin_user = models.User.objects.get(name="Administrator1")
        client_group = models.Group.objects.get(name="client")
        client_user = models.User.objects.get(name="Client1")
        self.assertEqual(admin_user.group, admin_group)
        self.assertEqual(client_user.group, client_group)

    def test_auth_token(self):
        """[Testing] Authenticate with username and password"""
        admin_user = models.User.objects.get(name="Administrator1")

        serializer = AuthTokenSerializer(data={'username': "admin1@test.com", 'password': "admin"})
        serializer.is_valid(raise_exception=True)
        user = serializer.validated_data['user']
        
        self.assertEqual(admin_user, user)
        


"""
Django settings for mysite project.

Generated by 'django-admin startproject' using Django 3.1.1.

For more information on this file, see
https://docs.djangoproject.com/en/3.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.1/ref/settings/
"""

from pathlib import Path
import environ

env = environ.Env(
    # set casting, default value
    DEBUG=(bool, False)
)
# reading .env file
environ.Env.read_env()

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

print(env('SECRET_KEY'))
SECRET_KEY = env('SECRET_KEY')

DEBUG = env('DEBUG')

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'rest_framework',
    'rest_framework.authtoken',
    'rest_framework_swagger',
    'corsheaders',
    'myapp.apps.MyappConfig',
    'api_v1.apps.MyappConfig',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'corsheaders.middleware.CorsMiddleware',
]

ROOT_URLCONF = 'mysite.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'mysite.wsgi.application'

# Database
# https://docs.djangoproject.com/en/3.1/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    },
    # 'mysql': env.db(),
    # 'postgresql': env.db(),
}


# Password validation
# https://docs.djangoproject.com/en/3.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

AUTH_USER_MODEL = 'api_v1.User'

# Internationalization
# https://docs.djangoproject.com/en/3.1/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.1/howto/static-files/

STATIC_URL = '/static/'

STATIC_ROOT = '{}/staticfiles'.format(BASE_DIR)

MEDIA_URL = '/media/'

MEDIA_ROOT = '{}/mediafiles'.format(BASE_DIR)


REST_FRAMEWORK = {
    'PAGE_SIZE': 10,
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
    'DEFAULT_SCHEMA_CLASS': 'rest_framework.schemas.coreapi.AutoSchema',
    'DEFAULT_AUTHENTICATION_CLASSES': [
        'rest_framework.authentication.TokenAuthentication',
        'rest_framework.authentication.SessionAuthentication',
    ]
}

SWAGGER_SETTINGS = {
    'LOGIN_URL': 'rest_framework:login',
    'LOGOUT_URL': 'rest_framework:logout',
    'DOC_EXPANSION': 'list',
}

CORS_ORIGIN_WHITELIST = [
    "http://localhost:8080",
]



  
#!/usr/bin/env python3
import sys
from setuptools import find_packages, setup

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 5)

# This check and everything above must remain compatible with Python 2.7.
if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================
This version of Django REST Framework requires Python {}.{}, but you're trying
to install it on Python {}.{}.
This may be because you are using a version of pip that doesn't
understand the python_requires classifier. Make sure you
have pip >= 9.0 and setuptools >= 24.2, then try again:
    $ python -m pip install --upgrade pip setuptools
    $ python -m pip install djangorestframework
This will install the latest version of Django REST Framework which works on
your version of Python. If you can't upgrade your pip (or Python), request
an older version of Django REST Framework:
    $ python -m pip install "djangorestframework<3.10"
""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='django-rest-swagger-master',
    version='1.0.5',
    url='https://github.com/crazy-coding/django-rest-swagger-master',
    license='BSD',
    description='Django Template with Template, Restframework and Swagger.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Dennis Cho',
    author_email='dennischo122@gmail.com',  # SEE NOTE BELOW (*)
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    install_requires=["django>=2.2"],
    python_requires=">=3.5",
    zip_safe=False,
    classifiers=[
        # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP',
    ]
)